SystemOrganization addCategory: #Pharogenesis!
SystemOrganization addCategory: #'Pharogenesis-Tests'!

!OBCodeBrowser methodsFor: '*pharoGenesis-methodversions' stamp: 'MaxLeske 11/15/2010 23:59'!
browseGenesisMethodVersions
	<command>
	
	^ OBCmdBrowseGenesisMethodVersions! !

!OBCodeBrowser methodsFor: '*pharoGenesis-methodversions' stamp: 'MaxLeske 10/31/2010 21:18'!
obsoleteBrowseMethodVersions
	<obsolete>
	
	^ OBCmdBrowseMethodVersions! !

TestCase subclass: #MethodDataTests
	instanceVariableNames: 'changeRecord methodChangeRecord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis-Tests'!

!MethodDataTests methodsFor: 'running' stamp: 'MaxLeske 1/31/2011 12:23'!
setUp
	changeRecord := ChangeRecord new.
	methodChangeRecord := GCRMethod new.
	methodChangeRecord changeRecord: changeRecord.
	
	changeRecord 
		stamp: 'TPR 5/12/1998 22:48';
		instVarNamed: 'meta' put: false;
		instVarNamed: 'category' put: 'converting';
		instVarNamed: 'class' put: 'String'.
	methodChangeRecord instVarNamed: 'selector' put: 'asString:'.! !

!MethodDataTests methodsFor: 'tests' stamp: 'MaxLeske 1/31/2011 13:01'!
testMethodDataCreation
	| parentName dataTemplate methodData |
	parentName := 'e1473fd845e629cb79f0126d8405d838136ab552'.
	
	dataTemplate := String streamContents: [ :stream |
		stream
			nextPutAll: 'author TPR';
			nextPutAll: String crlf;
			nextPutAll: 'creation_date ';
			nextPutAll: methodChangeRecord timeStamp printString;
			nextPutAll: String crlf;
			nextPutAll: 'parent_name ';
			nextPutAll: parentName;
			nextPutAll: String crlf ].
	
	methodData := methodChangeRecord methodData: parentName.
	self assert: methodData = dataTemplate.
	
	! !

!MethodDataTests methodsFor: 'tests' stamp: 'MaxLeske 1/31/2011 12:22'!
testMethodDataPath
	| targetReference referenceTemplate methodDataPath |
	targetReference := FSMemoryFilesystem new root.
	
	referenceTemplate :=
		targetReference /
		'String' /
		'instance' /
		'converting' /
		'asString-3A-_data.bin'.
		
	methodDataPath := methodChangeRecord methodDataPath: targetReference.
	self assert: methodDataPath = referenceTemplate! !

!MethodDataTests methodsFor: 'tests' stamp: 'MaxLeske 1/31/2011 13:07'!
testMethodDataReading
	| parentName dataFile dict stream reader creationDate |
	parentName := 'e1473fd845e629cb79f0126d8405d838136ab552'.
	dataFile := FSMemoryFilesystem new root / 'methodData.bin'.
	dataFile writeStreamDo: [ :writer | 
		writer nextPutAll: (methodChangeRecord methodData: parentName) ].
	
	dict := Dictionary new.
	stream := dataFile readStream.
	[ stream atEnd ] whileFalse: [
		reader := ReadStream on: (stream nextLine asString).
		dict
			at: (reader upTo: Character space)
			put: (reader upToEnd) ].
	
	creationDate := TimeStamp fromString: (dict at: 'creation_date').
	creationDate := creationDate offset: Duration zero.
	self assert: (dict at: 'author') = 'TPR'.
	self assert: creationDate = methodChangeRecord timeStamp.
	self assert: (dict at: 'parent_name') = parentName! !

!MethodDataTests methodsFor: 'tests' stamp: 'MaxLeske 1/31/2011 17:59'!
testmethodDataBlackBox
	| parentName targetReference dataFile methodData timeStamp directories |
	parentName := 'e1473fd845e629cb79f0126d8405d838136ab552'.
	targetReference := FSMemoryFilesystem new root.
	directories := targetReference / 'String' / 'instance' / 'converting'.
	directories ensureDirectory.
	dataFile := directories / 'asString-3A-_data.bin'.
	dataFile writeStreamDo: [ :writer | 
		writer nextPutAll: (methodChangeRecord methodData: parentName) ].
	
	methodData := Pharogenesis methodData: (methodChangeRecord objectPath: targetReference). 
	timeStamp := TimeStamp fromString: (methodData at: 'creation_date').
	timeStamp := timeStamp offset: Duration zero.
	self assert: (methodData at: 'author') = 'TPR'.
	self assert: timeStamp = methodChangeRecord timeStamp.
	self assert: (methodData at: 'parent_name') = parentName! !

TestCase subclass: #RepositoryCreationTests
	instanceVariableNames: 'repository source'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis-Tests'!

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 13:28'!
createSource
	source writeStreamDo: [ :stream |
		stream nextPutAll: self sourceContents ]! !

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 19:09'!
createdMethodPaths
	^ {
		self pathFrom: '/AbstractSound/instance/envelopes/envelopes.cs'.
		self pathFrom: '/AbstractSound/instance/envelopes/addEnvelope-3A-.cs'.
		self pathFrom: '/AbstractSound/instance/initialization/setPitch-3A-dur-3A-loudness-3A-.cs'.
		self pathFrom: '/AbstractSound/instance/initialization/setLoudness-3A-.cs'.
		self pathFrom: '/AbstractSound/instance/initialization/initialize.cs'.
		self pathFrom: '/AbstractSound/instance/initialization/duration-3A-.cs'.
		self pathFrom: '/AbstractScoreEvent/instance/all/time-3A-.cs'.
		self pathFrom: '/AbstractScoreEvent/instance/all/time.cs'.
		self pathFrom: '/AbstractScoreEvent/instance/all/isTempoEvent.cs'.
		self pathFrom: '/AbstractScoreEvent/instance/all/isNoteEvent.cs'. }! !

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 19:10'!
newMethodInCommit: anFSGitFilesystem at: anInteger
	| path |
	path := self createdMethodPaths at: anInteger.
	^ anFSGitFilesystem referenceTo: path! !

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 18:05'!
parentNameFor: aReference
	^ (Pharogenesis methodData: aReference) at: 'parent_name'
	  ! !

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 18:44'!
pathFrom: aString
	^ FSPath 
		readFrom: (ReadStream on: aString) 
		delimiter: $/! !

!RepositoryCreationTests methodsFor: 'running' stamp: 'MaxLeske 2/1/2011 13:28'!
setUp
	| target |
	source := FSDiskFilesystem current working / 'testChanges.changes'.
	self createSource.
	target := FSMemoryFilesystem new root.
	Pharogenesis new
		createRepositoryAt: target
		fromFiles: {source}.
	repository := FSGitRepository on: target / 'pharogenesis'.! !

!RepositoryCreationTests methodsFor: 'helpers' stamp: 'MaxLeske 2/1/2011 13:27'!
sourceContents
^
'''From Squeak 1.31 of Feb 4, 1998 on 4 February 1998 at 10:59:11 pm''!!


Abstract class for timed events in a MIDI score.
!!
!!AbstractScoreEvent methodsFor: ''all'' stamp: ''jm 12/31/97 11:46''!!
isNoteEvent

	^ false
!! !!

!!AbstractScoreEvent methodsFor: ''all'' stamp: ''jm 12/31/97 11:46''!!
isTempoEvent

	^ false
!! !!

!!AbstractScoreEvent methodsFor: ''all'' stamp: ''jm 12/31/97 11:43''!!
time

	^ time
!! !!

!!AbstractScoreEvent methodsFor: ''all'' stamp: ''jm 12/31/97 11:43''!!
time: aNumber

	time _ aNumber.
!! !!

!!AbstractSound methodsFor: ''initialization'' stamp: ''jm 12/9/97 11:31''!!
duration: seconds
	"Scale my envelopes to the given duration. Subclasses overriding this method should include a resend to super."

	envelopes do: [:e | e duration: seconds].
!! !!

!!AbstractSound methodsFor: ''initialization'' stamp: ''jm 2/4/98 09:54''!!
initialize

	envelopes _ #().
	mSecsSinceStart _ 0.
	samplesUntilNextControl _ 0.
	scaledVol _ (1.0 * ScaleFactor) rounded.
	scaledVolIncr _ 0.
	scaledVolLimit _ scaledVol.
!! !!

!!AbstractSound methodsFor: ''initialization'' stamp: ''jm 1/31/98 16:09''!!
setLoudness: vol
	"Initialize my volume envelopes and initial volume. Subclasses overriding this method should include a resend to super."

	envelopes do: [:e |
		(e isKindOf: VolumeEnvelope) ifTrue: [e scale: vol]].
	self initialVolume: vol.
!! !!

!!AbstractSound methodsFor: ''initialization'' stamp: ''jm 1/31/98 15:26''!!
setPitch: p dur: d loudness: l
	"Initialize my envelopes for the given parameters. Subclasses overriding this method should include a resend to super."

	envelopes do: [:e |
		(e isKindOf: VolumeEnvelope) ifTrue: [e scale: l].
		(e isKindOf: PitchEnvelope) ifTrue: [e centerPitch: p].
		e duration: d].
	self initialVolume: l.
	self duration: d.

!! !!

!!AbstractSound methodsFor: ''envelopes'' stamp: ''jm 12/17/97 22:23''!!
addEnvelope: anEnvelope
	"Add the given envelope to my envelopes list."

	anEnvelope target: self.
	envelopes _ envelopes copyWith: anEnvelope.
!! !!

!!AbstractSound methodsFor: ''envelopes'' stamp: ''jm 12/15/97 17:02''!!
envelopes
	"Return my collection of envelopes."

	^ envelopes
!! !!'! !

!RepositoryCreationTests methodsFor: 'running' stamp: 'MaxLeske 2/1/2011 13:28'!
tearDown
	source delete! !

!RepositoryCreationTests methodsFor: 'tests' stamp: 'MaxLeske 2/1/2011 14:00'!
testAllCommitsReachable
	| commit allCommits |
	allCommits := repository versions.
	self assert: allCommits size = 10.
	
	commit := repository head.	
	allCommits remove: commit.
	[ commit := commit parents first. 
		allCommits remove: commit ]
		doWhileFalse: [ commit parents isEmptyOrNil ].
		
	self assert: allCommits isEmpty! !

!RepositoryCreationTests methodsFor: 'tests' stamp: 'MaxLeske 2/1/2011 19:15'!
testMethodDataReferences
	| allCommits counter commit methodReference shouldBeParentName actualParentName |
	allCommits := repository versions. "youngest entry is versions first"
	1 to: allCommits size - 1 do: [ :index |
		commit := allCommits at: index.
		self assert: commit parents first = (allCommits at: index + 1) ].
	
	1 to: allCommits size - 1 do: [ :index |
		commit := allCommits at: index.
		methodReference := self newMethodInCommit: commit at: index.
		shouldBeParentName := commit parents first name.
		actualParentName := self parentNameFor: methodReference.
		self assert:  shouldBeParentName = actualParentName ]! !

TestCase subclass: #StampParsingTests
	instanceVariableNames: 'gcr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis-Tests'!

!StampParsingTests methodsFor: 'running' stamp: 'MaxLeske 10/30/2010 16:44'!
setUp
	gcr := GitChangeRecord new! !

!StampParsingTests methodsFor: 'tests' stamp: 'MML 9/15/2010 20:57'!
testDateFrom
	self assert: (gcr dateFrom: '10/4/97 00:08 di') = '10/4/1997'.
	self assert: (gcr dateFrom: '00:08 10/4/97 di') = '10/4/1997'.
	self assert: (gcr dateFrom: 'di 00:08 10/4/97') = '10/4/1997'.
	self assert: (gcr dateFrom: 'di 10/4/97 00:08') = '10/4/1997'.
	self assert: (gcr dateFrom: '00:08 di 10/4/97') = '10/4/1997'.
	self assert: (gcr dateFrom: '10/4/97 di 00:08') = '10/4/1997'.
	self assert: (gcr dateFrom: 'di 00:08') isEmpty! !

!StampParsingTests methodsFor: 'tests' stamp: 'MML 9/15/2010 20:58'!
testResolveStamp
	| template timeStamp |
	timeStamp := (TimeStamp fromString: '4/6/1997 00:08') offset: Duration zero.
	template := timeStamp asUnixTime asString , ' di'.
	
	self assert: (gcr resolveStamp: '4/6/97 00:08 di') = template! !

Object subclass: #GitChangeRecord
	instanceVariableNames: 'selector changeRecord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

GitChangeRecord subclass: #GCRClassComment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRClassComment class methodsFor: 'private' stamp: 'MML 9/15/2010 20:36'!
recordType
	^ #classComment! !

!GCRClassComment methodsFor: 'accessing' stamp: 'MaxLeske 11/5/2010 19:17'!
objectPath: targetReference
	^ (self basePath: targetReference) /
		'comment.txt'! !

GitChangeRecord subclass: #GCRDoIt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRDoIt class methodsFor: 'private' stamp: 'MML 9/15/2010 20:37'!
recordType
	^ #doIt! !

!GCRDoIt methodsFor: 'accessing' stamp: 'MaxLeske 1/11/2011 18:44'!
changeRecord: aChangeRecord
	| substrings |
	substrings := aChangeRecord string subStrings: ' '.
	substrings size < 2 ifTrue: [ 
		changeRecord := aChangeRecord. 
		^ self ].
	
	(substrings second = 'subclass:')
		ifTrue: [ ^ GCRDoItClassCreation new changeRecord: aChangeRecord ]
		ifFalse: [ ^ GCRDoItMetaClassCreation new changeRecord: aChangeRecord ]! !

!GCRDoIt methodsFor: 'internal-accessors' stamp: 'MaxLeske 9/16/2010 13:45'!
methodClassName: aString
	changeRecord instVarNamed: 'class' put: aString! !

!GCRDoIt methodsFor: 'internal-accessors' stamp: 'MaxLeske 9/16/2010 13:57'!
methodSelector: aString
	selector := aString! !

GCRDoIt subclass: #GCRDoItClassCreation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRDoItClassCreation methodsFor: 'accessing' stamp: 'MaxLeske 11/6/2010 08:53'!
changeRecord: aChangeRecord
	changeRecord := aChangeRecord! !

!GCRDoItClassCreation methodsFor: 'accessing' stamp: 'MaxLeske 1/11/2011 17:52'!
objectPath: targetReference
	^ (self basePath: targetReference) /
		'instance.st'! !

GCRDoIt subclass: #GCRDoItMetaClassCreation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRDoItMetaClassCreation methodsFor: 'accessing' stamp: 'MaxLeske 11/6/2010 08:53'!
changeRecord: aChangeRecord
	changeRecord := aChangeRecord! !

!GCRDoItMetaClassCreation methodsFor: 'accessing' stamp: 'MaxLeske 1/11/2011 17:52'!
objectPath: targetReference
	^ (self basePath: targetReference) /
		'class.st'! !

GitChangeRecord subclass: #GCRMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRMethod class methodsFor: 'private' stamp: 'MML 9/15/2010 20:42'!
recordType
	^ #method! !

!GCRMethod methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:16'!
instanceOrClassDir
	^ self isMetaClassChange
		ifTrue: [ 'class' ]
		ifFalse: [ 'instance' ]! !

!GCRMethod methodsFor: 'accessing' stamp: 'MaxLeske 1/31/2011 13:01'!
methodData: parentName
	^ String streamContents: [ :stream |
		stream 
			nextPutAll: 'author ';
			nextPutAll: self author;
			nextPutAll: String crlf;
			nextPutAll: 'creation_date ';
			nextPutAll: self timeStamp printString;
			nextPutAll: String crlf;
			nextPutAll: 'parent_name ';
			nextPutAll: parentName;
			nextPutAll: String crlf ]! !

!GCRMethod methodsFor: 'accessing' stamp: 'MaxLeske 1/31/2011 12:20'!
methodDataPath: targetReference
	^ (self basePath: targetReference) /
		self instanceOrClassDir /
		self validCategory /
		(self validMethodSelector , '_data.bin')! !

!GCRMethod methodsFor: 'accessing' stamp: 'MaxLeske 1/11/2011 17:52'!
objectPath: targetReference
	^ (self basePath: targetReference) /
		self instanceOrClassDir /
		self validCategory /
		self validMethodSelector , 'cs'
		! !

GitChangeRecord subclass: #GCRPreamble
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!GCRPreamble class methodsFor: 'private' stamp: 'MML 9/15/2010 20:42'!
recordType
	^ #preamble! !

!GitChangeRecord class methodsFor: 'initialization' stamp: 'MML 9/15/2010 20:39'!
from: aChangeRecord
	self allSubclassesDo: [ :each |
		aChangeRecord type = each recordType ifTrue: [ 
			^ each new changeRecord: aChangeRecord ] ]! !

!GitChangeRecord class methodsFor: 'private' stamp: 'MML 9/15/2010 20:33'!
recordType
	self subclassResponsibility! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:44'!
asMethodDefinition
	^ self changeRecord asMethodDefinition! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 11/1/2010 11:21'!
author
	| stamp |
	stamp := self stamp.
	(stamp beginsWith: 'invalid')
		ifTrue: [ ^ 'unknown' ]
		ifFalse: [ ^ (stamp subStrings: ' ') second ]! !

!GitChangeRecord methodsFor: 'private' stamp: 'MaxLeske 9/21/2010 17:36'!
basePath: targetReference
	^ targetReference /
		self methodClassName
	! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:43'!
category
	^ self changeRecord category! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 8/16/2010 16:26'!
changeRecord
	^ changeRecord! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 8/16/2010 16:26'!
changeRecord: aChangeRecord
	changeRecord := aChangeRecord! !

!GitChangeRecord methodsFor: 'parsing' stamp: 'MaxLeske 10/30/2010 11:47'!
dateFrom: aStamp
	| dateMatch year dateParts matches |
	matches := '\d+/\d+/\d+' asRegex matchesIn: aStamp.
	dateMatch := matches isEmpty
		ifTrue: [ nil ]
		ifFalse: [ matches first ].
	dateMatch isEmptyOrNil
		ifTrue: [ ^ '' ].
	dateParts := dateMatch subStrings: '/'.
	year := dateParts third.
	year size = 2
		ifTrue: [ year := '19' , year ].
	^ dateParts first , '/' , dateParts second , '/' , year! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:42'!
isMetaClassChange
	^ self changeRecord isMetaClassChange! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:41'!
methodClass
	^ self changeRecord methodClass! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:41'!
methodClassName
	^ self changeRecord methodClassName! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:58'!
methodSelector
	^ selector ifNil: [ self changeRecord methodSelector ]! !

!GitChangeRecord methodsFor: 'converting' stamp: 'MML 9/15/2010 21:06'!
mutator
	^ self recordType asMutator! !

!GitChangeRecord methodsFor: 'parsing' stamp: 'MaxLeske 11/1/2010 11:28'!
nameFrom: aStamp
	| result |
	result := ''.
	aStamp regex: '[a-zA-Z]+' matchesDo: [ :match | match size > result size ifTrue: [ result := match ] ].
	^ result ifEmpty: [ 'unknown' ]
	
	! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:08'!
objectPath: targetReference
	self subclassResponsibility! !

!GitChangeRecord methodsFor: 'private' stamp: 'MML 9/15/2010 21:06'!
recordType
	^ self class recordType! !

!GitChangeRecord methodsFor: 'parsing' stamp: 'MaxLeske 11/3/2010 09:09'!
resolveStamp: aStamp
	"stamp format: 'TPR 5/12/1998 22:48', '6/11/97 09:18 di'"
	
	|  name date time timeStamp |
	aStamp isEmptyOrNil ifTrue: [ ^ 'invalid stamp' ].
	
	date := self dateFrom: aStamp.
	name := self nameFrom: aStamp.
	time := self timeFrom: aStamp.
	(date isEmpty or: [ time isEmpty ])
		ifTrue: [ ^ '0 ' , name ].	
	timeStamp := (TimeStamp fromString: (date , ' ' , time)) offset: Duration zero.
	
	timeStamp asUnixTime negative
		ifTrue: [ ^ '0 ', name ]
		ifFalse: [ ^ timeStamp asUnixTime asString , ' ' , name ]! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MML 9/15/2010 21:08'!
stamp
	^ self resolveStamp: self changeRecord stamp! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:39'!
string
	^ self changeRecord string! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:39'!
text
	^ self changeRecord text! !

!GitChangeRecord methodsFor: 'parsing' stamp: 'MML 9/15/2010 20:25'!
timeFrom: aStamp
	| result |
	result := aStamp allRegexMatches: '\d\d\:\d\d'.
	result isEmptyOrNil ifTrue: [ ^ '' ].
	^ result first
	
	! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 11/1/2010 11:21'!
timeStamp
	| unixTime stamp |
	stamp := self stamp.
	(stamp beginsWith: 'invalid')
		ifTrue: [ unixTime := 0 ]
		ifFalse: [
			unixTime := (self stamp subStrings: ' ') first asInteger ].
	
	^ (TimeStamp fromUnixTime: unixTime) offset: Duration zero! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/14/2010 21:38'!
type
	^ self changeRecord type! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:10'!
validCategory
	^ self validNodeName: self category! !

!GitChangeRecord methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:10'!
validMethodSelector
	^ self validNodeName: self methodSelector! !

!GitChangeRecord methodsFor: 'parsing' stamp: 'MaxLeske 11/8/2010 15:00'!
validNodeName: aSymbol
	^ (((aSymbol asString
		copyReplaceAll: '-' with: '-2D-')
		copyReplaceAll: ':' with: '-3A-')
		copyReplaceAll: '/' with: '-2F-')
		copyReplaceAll: '\' with: '-5C-'! !

Object subclass: #Pharogenesis
	instanceVariableNames: 'target searcher parser gitFS repository parentName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Pharogenesis'!

!Pharogenesis class methodsFor: 'convenience' stamp: 'MaxLeske 1/16/2011 17:37'!
convertSelector: aSymbol
	^ (((aSymbol asString
		copyReplaceAll: '-' with: '-2D-')
		copyReplaceAll: ':' with: '-3A-')
		copyReplaceAll: '/' with: '-2F-')
		copyReplaceAll: '\' with: '-5C-'! !

!Pharogenesis class methodsFor: 'public' stamp: 'MaxLeske 1/30/2011 20:10'!
createRepositoryAt: targetReference fromFilesIn: sourceDirectoryReference
	self new
		createRepositoryAt: targetReference
		fromFilesIn: sourceDirectoryReference! !

!Pharogenesis class methodsFor: 'convenience' stamp: 'MaxLeske 1/31/2011 17:59'!
methodData: methodReference
	| selector dataStream entries reader |
	selector := methodReference basename withoutPeriodSuffix.
	dataStream := (methodReference parent / (selector, '_data.bin')) readStream.
	entries := Dictionary new.

	[ dataStream atEnd ] whileFalse: [
		reader := ReadStream on: (dataStream nextLine asString).
		entries
			at: (reader upTo: Character space)
			put: (reader upToEnd) ].
		
	^ entries! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 10/30/2010 13:31'!
changeRecordsFor: aReference do: aBlock displayingProgress: aString
	| changes |
	changes := FileStream oldFileNamed: aReference path asString do: [ :sourceStream | 
		sourceStream converter: Latin1TextConverter new.
		ChangeSet scanFile: sourceStream from: 0 to: sourceStream size ].
	
	aString isEmptyOrNil
		ifTrue: [
			changes 
				do: [ :change | 
					aBlock value: (GitChangeRecord from: change) ] ]
		ifFalse: [
			changes 
				do: [ :change | 
					aBlock value: (GitChangeRecord from: change) ] 
				displayingProgress: aString ]! !

!Pharogenesis methodsFor: 'record types' stamp: 'MaxLeske 11/1/2010 13:33'!
classComment: aChangeRecord
	self 
		write: aChangeRecord string 
		to: (aChangeRecord objectPath: target)! !

!Pharogenesis methodsFor: 'parsing' stamp: 'MaxLeske 7/27/2010 12:19'!
classCreation
	^ '`@superclass `subclass: `#subclass
		instanceVariableNames: `#instVars
		classVariableNames: `#classVars
		poolDictionaries: `#poolDicts
		category: `#category'! !

!Pharogenesis methodsFor: 'parsing' stamp: 'MaxLeske 1/16/2011 17:35'!
convert: aString
	^ (aString asString 
		copyReplaceAll: (String with: Character cr with: Character lf) with: (String with: Character lf))
		copyReplaceAll: (String with: Character cr) with: (String with: Character lf)! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 11/4/2010 23:08'!
copySource: sourceReference to: targetReference
	target := targetReference.
	
	self 
		changeRecordsFor: sourceReference
		do:[ :change |
			self processChange: change ]
		displayingProgress: 'Processing ' , sourceReference basename surroundedBySingleQuotes , '...'! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 1/31/2011 11:40'!
createRepositoryAt: targetReference fromFiles: fileReferences
	repository := FSGitRepository on: targetReference / 'pharogenesis'.
	gitFS := repository head.
	gitFS basicSwitchToBranch: 'squeak'.
	target := gitFS root.
	parentName := ''.
	
	fileReferences do: [ :file |
		(file basename endsWith: '.sources')
			ifTrue: [ self singleCommitSources: file  ]
			ifFalse: [ self incrementallyCommitChanges: file ] ].! !

!Pharogenesis methodsFor: 'public' stamp: 'MaxLeske 1/16/2011 15:38'!
createRepositoryAt: targetReference fromFilesIn: sourceDirectoryReference
	| files |
	files := self namesOfSources collect: [ :name || ref |
		ref := sourceDirectoryReference / name. 
		ref exists not ifTrue: [ FSFileDoesNotExist signalWith: ref ].
		ref ].
	
	self
		createRepositoryAt: targetReference
		fromFiles: files! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 1/11/2011 18:46'!
deleteSelector: selector fromClass: className
	| reference encodedSelector|
	encodedSelector := self convert: selector.
	reference := gitFS root / className.
	reference := reference allChildren 
		detect: [ :ref |
			ref basename = (encodedSelector , 'cs') ]
		ifNone: [ nil ].
	reference ifNotNil: [ reference delete ]! !

!Pharogenesis methodsFor: 'record types' stamp: 'MaxLeske 11/5/2010 20:15'!
doIt: aChangeRecord
	| tree |
	self searcher initialize.
	tree := RBParser parseExpression: aChangeRecord string
			onError: [ :msg :position | ^ nil ].

	self searcher 
		matches: self classCreation
		do: [ :node :answer |
			aChangeRecord methodClassName: node arguments first value asString.
			self 
				write: node formattedCode 
				to:  (aChangeRecord objectPath: target) ];
		
		matches: self metaClassCreation
		do: [ :node :answer |
			aChangeRecord methodClassName: node receiver receiver name asString.
			self 
				write: node formattedCode 
				to: (aChangeRecord objectPath: target) ];
		
		matches: self methodRemoval
		do: [ :node :answer |
			aChangeRecord methodSelector: node arguments first value asString.
			aChangeRecord methodClassName: node receiver name asString.
			^ self 
				deleteSelector: aChangeRecord methodSelector
				fromClass: aChangeRecord methodClassName ];
		executeTree: tree.
	
	tree isMessage ifTrue: [ ^ self ].
	tree isSequence ifTrue: [ ^ self ].
	"self halt"! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 2/1/2011 19:22'!
incrementallyCommitChanges: aReference
	self
		changeRecordsFor: aReference
		do: [ :change |
			self processChange: change.
			self setStampFrom: change.
			gitFS commit: 'no message'.
			parentName := gitFS name ]
		displayingProgress: 'creating commits for ', aReference basename, '...'.
		
	gitFS 
		tagRevision: gitFS parents first 
		with: aReference basename! !

!Pharogenesis methodsFor: 'parsing' stamp: 'MaxLeske 7/27/2010 12:20'!
metaClassCreation
	^ '`class class instanceVariableNames: `#instVars'! !

!Pharogenesis methodsFor: 'record types' stamp: 'MaxLeske 1/31/2011 11:37'!
method: aChangeRecord
	self 
		write: aChangeRecord string 
		to: (aChangeRecord objectPath: target).
	self 
		write: (aChangeRecord methodData: parentName)
		to: (aChangeRecord methodDataPath: target)! !

!Pharogenesis methodsFor: 'parsing' stamp: 'MaxLeske 7/27/2010 12:21'!
methodRemoval
	^ '`class removeSelector: `#selector'! !

!Pharogenesis methodsFor: 'accessing' stamp: 'MaxLeske 1/31/2011 21:36'!
namesOfSources
	^ #(
		'SqueakV1.sources'
		'Squeak1.31.changes'
		'SqueakV2.sources'
		'Squeak2.8.changes'
		'SqueakV3.sources'
		'Squeak3.8.1-6747full.changes'
		'SqueakV39.sources'
		'Squeak3.9.1-final-7075.changes'
		'SqueakV40.sources'
		'PharoV10.sources'
	)! !

!Pharogenesis methodsFor: 'record types' stamp: 'MaxLeske 7/26/2010 20:21'!
preamble: aChangeRecord
	"ignore".! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 10/30/2010 13:38'!
processChange: aChangeRecord
	self
		perform: aChangeRecord mutator
		with: aChangeRecord! !

!Pharogenesis methodsFor: 'accessing' stamp: 'MaxLeske 9/16/2010 13:33'!
searcher
	^ searcher ifNil: [ searcher := RBParseTreeSearcher new ]! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 11/1/2010 10:45'!
setStampFrom: aChange
	| stamp |
	stamp := GitStamp new
		name: aChange author;
		email: 'unknown';
		timeStamp: aChange timeStamp;
		yourself.
		
	gitFS
		setAuthor: stamp;
		setCommitter: stamp! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 2/1/2011 19:31'!
singleCommitSources: sourceReference
	| subStrings author committer |
	(sourceReference basename beginsWith: 'Pharo')
		ifTrue: [ gitFS := repository tagNamed: 'Squeak3.9.1-final-7075.changes' ].
	
	gitFS detachWorkingCopy.
	self 
		copySource: sourceReference
		to: gitFS root.
	
	author := GitStamp new
		name: 'unknown';
		email: 'unknown';
		yourself.
	committer := GitStamp new
		name: 'Max Leske';
		email: 'maxleske@gmail.com'
		yourself.
	gitFS
		setAuthor: author;
		setCommitter: committer.	
	
	(sourceReference basename beginsWith: 'Pharo')
		ifTrue: [
			gitFS
				branch: 'pharo'
				message: 'Pharo branch; single commit for ', sourceReference basename
				andTag: sourceReference basename.
			parentName := gitFS name ]
		ifFalse: [
			parentName := gitFS name.
			gitFS
				commit: 'single commit for ', sourceReference basename
	 			andTag: sourceReference basename ]! !

!Pharogenesis methodsFor: 'private' stamp: 'MaxLeske 11/5/2010 19:13'!
write: aString to: aReference
	aString species = WideString ifTrue: [ ^ self ].
	
	aReference parent ensureDirectory.
	aReference writeStreamDo: [ :stream | stream nextPutAll: (self convert: aString) ]! !
